@@ -6,13 +6,14 @@ import sys
 from io import BytesIO
 import time
 import os
+from typing import Union, Dict, Optional
 
 import matplotlib.figure
 import matplotlib.pyplot as plt
 import networkx as nx
 # ML dependencies
 import numpy as np
-from multiprocess.context import Process
+from multiprocessing.context import Process
 from osbrain import Agent
 from osbrain import NSProxy
 from osbrain import run_agent
@@ -21,10 +22,10 @@ from plotly import tools as tls
 from sklearn.model_selection import ParameterGrid
 import copy
 
-from agentMET4FOF.dashboard.Dashboard import AgentDashboard as AgentDashboard
-from agentMET4FOF.streams import DataStreamMET4FOF
+from .dashboard.Dashboard import AgentDashboard as AgentDashboard
+from .streams import DataStreamMET4FOF
 
-from agentMET4FOF.develop.ML_Experiment import save_experiment
+from .develop.ML_Experiment import save_experiment
 
 
 class AgentMET4FOF(Agent):
@@ -139,15 +140,15 @@ class AgentMET4FOF(Agent):
         except Exception as e:
                 return -1
 
-    def init_agent_loop(self, loop_wait=1.0):
+    def init_agent_loop(self, loop_wait: Optional[int] = 1.0):
         """
-        Initiates the agent loop, which iterates every`loop_wait` seconds
+        Initiates the agent loop, which iterates every `loop_wait` seconds
 
         Stops every timers and initiate a new loop.
 
         Parameters
         ----------
-        loop_wait : int
+        loop_wait : int, optional
             The wait between each iteration of the loop
         """
         self.loop_wait = loop_wait
@@ -188,6 +189,9 @@ class AgentMET4FOF(Agent):
         """
         return message
 
+    @property
+    def buffer_filled(self):
+        return len(self.memory[self.name][next(iter(self.memory[self.name]))]) >= self.buffer_size
 
     def pack_data(self,data, channel='default'):
         """
@@ -408,7 +412,7 @@ class AgentMET4FOF(Agent):
             if self.log_mode:
                 self.log_info("Disconnected output module: "+ module_id)
 
-    def convert_to_plotly(self, matplotlib_fig):
+    def _convert_to_plotly(self, matplotlib_fig: matplotlib.figure.Figure):
         """
         Internal method to convert matplotlib figure to plotly figure
 
@@ -425,7 +429,7 @@ class AgentMET4FOF(Agent):
         return plotly_fig
 
 
-    def _fig_to_uri(self, matplotlib_fig = plt.figure()):
+    def _fig_to_uri(self, matplotlib_fig : matplotlib.figure.Figure):
         """
         Internal method to convert matplotlib figure to base64 uri image for display
 
@@ -443,36 +447,59 @@ class AgentMET4FOF(Agent):
         encoded = base64.b64encode(out_img.read()).decode("ascii").replace("\n", "")
         return "data:image/png;base64,{}".format(encoded)
 
-    def send_plot(self, fig=plt.Figure()):
+    def send_plot(self, fig: Union[matplotlib.figure.Figure, Dict[str,matplotlib.figure.Figure]], mode:str ="image"):
         """
         Sends plot to agents connected to this agent's Output channel.
 
         This method is different from send_output which will be sent to through the
         'plot' channel to be handled.
 
+        Tradeoffs between "image" and "plotly" modes are that "image" are more stable and "plotly" are interactive.
+        Note not all (complicated) matplotlib figures can be converted into a plotly figure.
+
         Parameters
         ----------
 
-        fig : Figure
-            Can be either matplotlib figure or plotly figure
+        fig : matplotlib.figure.Figure or dict of matplotlib.figure.Figure
+            Alternatively, multiple figures can be nested in a dict (with any preferred keys) e.g {"Temperature":matplotlib.Figure, "Acceleration":matplotlib.Figure}
+
+        mode : str
+            "image" - converts into image via encoding at base64 string.
+            "plotly" - converts into plotly figure using `mpl_to_plotly`
+            Default: "image"
 
         Returns
         -------
-        The message format is {'from':agent_name, 'plot': data, 'senderType': agent_class}.
+
+        graph : str or plotly figure or dict of one of those converted figure(s)
+
         """
+
+        error_msg = "Conversion mode "+mode+" is not implemented."
+
         if isinstance(fig, matplotlib.figure.Figure):
-            #graph = self._convert_to_plotly(fig) #unreliable
-            graph = self._fig_to_uri(fig)
+            if mode == "plotly":
+                graph = self._convert_to_plotly(fig)
+            elif mode == "image":
+                graph = self._fig_to_uri(fig)
+            else:
+                raise NotImplementedError(error_msg)
         elif isinstance(fig, dict): #nested
-            for key in fig.keys():
-                fig[key] = self._fig_to_uri(fig[key])
+            if mode == "plotly":
+                for key in fig.keys():
+                    fig[key] = self._convert_to_plotly(fig[key])
+            elif mode == "image":
+                for key in fig.keys():
+                    fig[key] = self._fig_to_uri(fig[key])
+            else:
+                raise NotImplementedError(error_msg)
             graph = fig
-        else:
+        else: #a plotly figure
             graph = fig
         self.send_output(graph, channel="plot")
         return graph
 
-    def update_data_memory(self,message):
+    def update_data_memory(self,agent_from,data=None):
         """
         Updates data stored in `self.memory` with the received message
 
@@ -481,10 +508,19 @@ class AgentMET4FOF(Agent):
 
         Parameters
         ----------
-        message : dict
-            Standard message format specified by AgentMET4FOF class
+        agent_from : dict | str
+            if type is dict, we expect it to be the agentMET4FOF dict message to be compliant with older code
+            otherwise, we expect it to be name of agent sender and `data` will need to be passed as parameter
+        data
+            optional if agent_from is a dict. Otherwise this parameter is compulsory. Any supported data which can be stored in dict as buffer.
 
         """
+        # if first argument is the agentMET4FOF dict message
+        if isinstance(agent_from, dict):
+            message = agent_from
+        # otherwise, we expect the name of agent_sender and the data to be passed
+        else:
+            message = {"from":agent_from, "data":data}
 
         # check if sender agent has sent any message before:
         # if it did,then append, otherwise create new entry for it
